/*
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 /

/*
 *  ======== LoggingSetup.xdc ========
 */

package ti.uia.sysbios;

/*!
 *  ======== LoggingSetup ========
 *  Module to aid in configuring SYSBIOS logging using UIA and System Analyzer
 * @p
 * The LoggingSetup module automates the process of configuring an application
 * to use UIA events, and configures SYS/BIOS modules to capture user-specified
 * information such as CPU Load, Task Load and Task Execution so that it can
 * be displayed by System Analyzer.  It also automates the creation of
 * infrastructure modules such as loggers, the ServiceManager and RTA modules
 * to enable the capture and upload of the events over a user-specified transport.
 * Both JTAG and Non-JTAG transports are supported.
 * @p
 *  The following configuration script demonstrates the use of the LoggingSetup
 *  module in the XDC configuration file for the application:
 *
 * @a(Example)
 * Example 1: Configuring an application to use the default settings provided
 * by LoggingSetup.  The following default settings are automatically applied:
 * @p(blist)
 *    - Logging UIA events from user-provided C code.  User provided C code is
 *    treated as part of the xdc.runtime.Main module.  A circular buffer of
 *    size {@link #mainLoggerSize} Bytes is enabled by default to support
 *    this.
 *   - Event logging is enabled for the SYS/BIOS Load and Task modules in order
 *   to allow System Analyzer to display CPU Load, Task Load, and Task Execution
 *   information.  Logging of SWI and HWI events is disabled by default.
 *   In order to optimize event capture and minimize event loss, two loggers
 *   are created: one to store events from the SYS/BIOS Load module
 *   and the other to store events from other SYS/BIOS modules.
 *   - Multicore event correlation is enabled by default.  This enables the
 *    {@link ti.uia.runtime.LogSync LogSync} module and creation of a dedicated
 *    logger for sync point events with a default circular buffer size of
 *    {@link ti.uia.runtime.LogSync#defaultSyncLoggerSize LogSync.defaultSyncLoggerSize} Bytes.
 *   - The Event Upload Mode is configured for UploadMode_JTAGSTOPMODE.  This
 *    allows events to be uploaded when the target halts via JTAG.
 *  @p
 *  @p(code)
 *  // the Log module provides logging APIs for use by the user's software
 *  var Log = xdc.useModule('xdc.runtime.Log');
 *  // the LoggingSetup module's default settings configure much of the UIA infrastructure.
 *  var LoggingSetup = xdc.useModule('ti.uia.sysbios.LoggingSetup');
 *  @p
 *  @p(html)
 *  <hr />
 *  @p
 * @a(Example)
 * Example 2: A number of 'template' applications come with the UIA product, that
 * provide predefined XDC configuration scripts and C code for use in new
 * projects.  These templates provide good examples of how to configure
 * the various modules that are involved in setting up the UIA infrastructure,
 * including the LoggingSetup module.
 * They can also be generated directly by CCS, using the CCS TI Resource
 * Explorer.  The following steps show how to use CCSv5.x.x to generate a
 * new project that configures the NDK to provide an Ethernet transport for
 * uploading events from the target to the host. :
 * @p(blist)
 *   - View / TI Resource Explorer : Opens the TI Resource Explorer.
 *   - Expand the System Analyzer (UIA Target) section
 *   - Expand the C6000, C64xx Multicore DSP, and EVMC6472 sections.
 *   - Expand the Single-core Examples and select Stairstep Ethernet.
 *   - Import the Stairstep Ethernet example into CCS.
 *   - Open the configuration file to see how LoggingSetup is configured
 *   with the NDK to use Ethernet as a transport for UIA events and
 *   commands.
 */
metaonly module LoggingSetup
{

    /*!
     *  ======== UploadMode ========
     *  Modes for uploading events to the host.
     *
     *  By default, events are uploaded using the JTAG connection in
     *  stop-mode. Events are uploaded over JTAG when the target halts. This
     *  mode requires that JTAG connections are supported by the target.
     *  If you want to use Ethernet, probe points, a simulator, or some other
     *  method for uploading events, you can specify one of those upload
     *  methods by configuring the  {@link #eventUploadMode} parameter.
     *  For example, you could use the following if you were running a
     *  simulator:
     *  @p(code)
     *  var LoggingSetup =
     *  xdc.useModule('ti.uia.sysbios.LoggingSetup');
     *  LoggingSetup.eventUploadMode = LoggingSetup.UploadMode_SIMULATOR;
     *  @p
     *
     *  @c(UploadMode_SIMULATOR)
     *  Simulator. Events are uploaded when logged. The target is briefly
     *  halted when the event is uploaded. This is equivalent to using
     *  UploadMode_PROBEPOINT.
     *
     *  @c(UploadMode_PROBEPOINT)
     *  Probepoint. Events are uploaded over JTAG when logged. The target is briefly
     *  halted when the event is uploaded. This mode can also be used
     *  with a simulator.
     *
     *  @c(UploadMode_JTAGSTOPMODE)
     *  Events are uploaded over JTAG when the target halts. This mode can
     *  also be used with a simulator.
     *
     *  @c(UploadMode_JTAGRUNMODE)
     *  Events are Uploaded over JTAG while the target is running. This mode is
     *  supported for C6x devices only.
     *
     *  @c(UploadMode_NONJTAGTRANSPORT)
     *  Events are uploaded over the non-JTAG transport specified by the
     *  {@link ti.uia.runtime.ServiceMgr#transportType ServiceMgr.transportType}
     *  parameter while the target is running.
     *
     *  @c(UploadMode_NONJTAG_AND_JTAGSTOPMODE)
     *  Events are uploaded over a non-JTAG transport specified by the
     *  {@link ti.uia.runtime.ServiceMgr#transportType ServiceMgr.transportType}
     *  parameter while the target is running. When the target is halted,
     *  events are uploaded over JTAG.
     *
     *  @c(UploadMode_STREAMER)
     *  Events are logged to a {@link LoggerStreamer} logger.
     *  The application must manage the buffers sent to the host.
     *
     *  @c(UploadMode_STREAMER2)
     *  Events are logged to {@link LoggerStreamer2} loggers.
     *  The application must manage the buffers sent to the host.
     *
     *  @c(UploadMode_IDLE)
     *  Events are logged to the {@link LoggerIdle} buffer, and uploaded in
     *  the idle loop. The user must specify the
     *  {@link LoggerIdle#transportType} and provide the
     *  {@link LoggerIdle#transportFxn}.
     */
    enum UploadMode {
        UploadMode_SIMULATOR = 1,    /*! Simulator. */
        UploadMode_PROBEPOINT = 2,   /*! Probepoint. */
        UploadMode_JTAGSTOPMODE = 3, /*! JTAG Stop Mode. */
        UploadMode_JTAGRUNMODE = 4,  /*! JTAG Run Mode (C6x only). */
        UploadMode_NONJTAGTRANSPORT = 5, /*! Non-JTAG transport */
        UploadMode_NONJTAG_AND_JTAGSTOPMODE = 6, /*! Non-JTAG transport and
                                                  *  JTAG Stop Mode. */
        UploadMode_STREAMER = 7,      /*! LoggerStreamer */
        UploadMode_IDLE = 8,          /*! Upload in Idle loop */
        UploadMode_STREAMER2 = 9      /*! LoggerStreamer2 */
    }

    /*!
     *  ======== loadLogger ========
     *  Logger used for the Load module Log events
     */
    config xdc.runtime.ILogger.Handle loadLogger = null;

    /*!
     *  ======== loadLoggerSize ========
     *  Size (in MAUs) of logger used for the Load module Log events.
     */
    metaonly config SizeT loadLoggerSize = 512;

    /*!
     *  ======== loadLogging ========
     *  Enable the Load module event logging.
     *
     *  If this is false, the events will be disabled. Otherwise the events
     *  will be enabled.
     *  Use the {@link #loadLoggingRuntimeControl} parameter
     *  to determine whether the state can be modified during runtime.
     */
    metaonly config Bool loadLogging = true;

    /*!
     *  ======== loadLoggingRuntimeControl ========
     *  Specify whether load logging can be enabled / disabled at runtime.
     *
     *  This determines what diags settings are applied to the module's diags
     *  mask. If 'false', the diags bits will be configured as
     *  ALWAYS_ON, meaning they can't be changed at runtime. If 'true', the
     *  bits will be configured as 'RUNTIME_ON'.
     *
     *  Use the {@link #loadLogging} parameter
     *  to determine whether the event is ON or OFF. For example, the
     *  following two lines set the Load module's events to 'ALWAYS_ON'.
     *
     *  @p(code)
     *  LoggingSetup.loadLogging = true;
     *  LoggingSetup.loadLoggingRuntimeControl = false;
     *  @p
     */
    metaonly config Bool loadLoggingRuntimeControl = true;

    /*!
     *  ======== mainLogger ========
     *  Logger used for main and non-XDC modules Log events
     */
    config xdc.runtime.ILogger.Handle mainLogger = null;

    /*!
     *  ======== mainLoggerSize ========
     *  Size (in MAUs) of logger used for the main and non-XDC modules Log events
     */
    metaonly config SizeT mainLoggerSize = 1024;

    /*!
     *  ======== mainLogging ========
     *  Enable main and non-XDC modules event logging
     *
     *  If this is false, the events will be disabled. Otherwise the events
     *  will be enabled. Use the {@link #mainLoggingRuntimeControl} parameter
     *  to determine whether the state can be modified during runtime.
     *
     *  The table below shows the initial values of the
     *  xdc.runtime.Main Diags mask settings, based on mainLogging and
     *  mainLoggingRuntimeControl configuration. These settins only
     *  apply to Diags masks that have not been set in the application's
     *  configuration file.  For example, this configuration code would
     *  cause xdc.runtime.Main's USER1 Diags mask to be unaffected by
     *  the settings of mainLogging and mainLoggingRuntimeControl:
     *
     *  @p(code)
     *  Diags.setMaskMeta('xdc.runtime.Main', Diags.USER1, Diags.RUNTIME_ON);
     *  @p
     *
     *  @p(html)
     *  <h3> Diags Mask Settings for xdc.runtime.Main </h3>
     *  <table border="1" cellpadding="3">
     *    <colgroup span="1"></colgroup> <colgroup span="5" align="center">
     *  </colgroup>
     *    <tr><th colspan="2" rowspan="2">  </th><th colspan="2">  mainLoggingRuntimeControl </th></tr>
     *    <tr><th>  true   </th><th> false </th></tr>
     *    <tr><th rowspan="2">  mainLogging  </th><th>  true  </th><td> RUNTIME_ON</td>
     *    <td> ALWAYS_ON </td></tr>
     *    <tr><th> false </th><td>  RUNTIME_OFF  </td> <td> ALWAYS_OFF </td></tr>
     * </table>
     */
    metaonly config Bool mainLogging = true;

    /*!
     *  ======== mainLoggingRuntimeControl ========
     *  Specify whether main logging can be enabled / disabled at runtime.
     *
     *  This determines what diags settings are applied to the module's diags
     *  mask. If 'false' and {@link #mainLogging} is true, the diags bits will be configured as
     *  ALWAYS_ON, meaning they can't be changed at runtime. If 'true', the
     *  bits will be configured as 'RUNTIME_ON'.
     *
     *  Use the {@link #mainLogging} parameter
     *  to determine whether the event is ON or OFF. For example, the
     *  following two lines set the xdc.runtime.Main events to
     *  initially be 'RUNTIME_ON'.
     *
     *  @p(code)
     *  LoggingSetup.mainLogging = true;
     *  LoggingSetup.mainLoggingRuntimeControl = true;
     *  @p
     */
    metaonly config Bool mainLoggingRuntimeControl = true;

    /*!
     *  ======== sysbiosLogger ========
     *  Logger used for SYSBIOS modules Log events
     */
    config xdc.runtime.ILogger.Handle sysbiosLogger = null;

    /*!
     *  ======== sysbiosLoggerSize ========
     *  Size (in MAUs) of the logger used for the SYS/BIOS modules' Log events
     */
    metaonly config SizeT sysbiosLoggerSize = 1024;

    /*!
     *  ======== sysbiosHwiLogging ========
     *  Enable SYSBIOS Hwi and Clock modules' event logging
     *
     *  If this is false, the events will be disabled. Otherwise the events
     *  will be enabled.
     *  Use the {@link #sysbiosHwiLoggingRuntimeControl} parameter
     *  to determine whether the state can be modified during runtime.
     */
    metaonly config Bool sysbiosHwiLogging = false;

    /*!
     *  ======== sysbiosHwiLoggingRuntimeControl ========
     *  Specify whether Hwi and Clock logging can be enabled / disabled at runtime
     *
     *  This determines what diags settings are applied to the module's diags
     *  mask. If 'false', the diags bits will be configured as
     *  ALWAYS_ON, meaning they can't be changed at runtime. If 'true', the
     *  bits will be configured as 'RUNTIME_ON'.
     *
     *  Use the {@link #sysbiosHwiLogging} parameter
     *  to determine whether the event is ON or OFF. For example, the
     *  following two lines set the Hwi module's events to 'ALWAYS_ON'.
     *
     *  @p(code)
     *  LoggingSetup.sysbiosHwiLogging = true;
     *  LoggingSetup.sysbiosHwiLoggingRuntimeControl = false;
     *  @p
     */
    metaonly config Bool sysbiosHwiLoggingRuntimeControl = false;

    /*!
     *  ======== sysbiosSwiLogging ========
     *  Enable SYSBIOS Swi module's event logging
     *
     *  If this is false, the events will be disabled. Otherwise the events
     *  will be enabled.
     *  Use the {@link #sysbiosSwiLoggingRuntimeControl} parameter
     *  to determine whether the state can be modified during runtime.
     */
    metaonly config Bool sysbiosSwiLogging = false;

    /*!
     *  ======== sysbiosSwiLoggingRuntimeControl ========
     *  Specify whether Swi logging can be enabled / disabled at runtime.
     *
     *  This determines what diags settings are applied to the module's diags
     *  mask. If 'false', the diags bits will be configured as
     *  ALWAYS_ON, meaning they can't be changed at runtime. If 'true', the
     *  bits will be configured as 'RUNTIME_ON'.
     *
     *  Use the {@link #sysbiosSwiLogging} parameter
     *  to determine whether the event is ON or OFF. For example, the
     *  following two lines initialize the Swi module's events to
     *  'RUNTIME_OFF'.
     *
     *  @p(code)
     *  LoggingSetup.sysbiosSwiLogging = false;
     *  LoggingSetup.sysbiosSwiLoggingRuntimeControl = true;
     *  @p
     */
    metaonly config Bool sysbiosSwiLoggingRuntimeControl = false;

    /*!
     *  ======== sysbiosTaskLogging ========
     *  Enable SYSBIOS Task module's event logging
     *
     *  If this is false, the events will be disabled. Otherwise the events
     *  will be enabled.
     *  Use the {@link #sysbiosTaskLoggingRuntimeControl} parameter
     *  to determine whether the state can be modified during runtime.
     */
    metaonly config Bool sysbiosTaskLogging = true;

    /*!
     *  ======== sysbiosTaskLoggingRuntimeControl ========
     *  Specify whether Task logging can be enabled / disabled at runtime.
     *
     *  This determines what diags settings are applied to the module's diags
     *  mask. If 'false', the diags bits will be configured as
     *  ALWAYS_ON, meaning they can't be changed at runtime. If 'true', the
     *  bits will be configured as 'RUNTIME_ON'.
     *
     *  Use the {@link #sysbiosTaskLogging} parameter
     *  to determine whether the event is ON or OFF. For example, the
     *  following two lines initialize the Task module's events to
     *  'ALWAYS_ON'.
     *
     *  @p(code)
     *  LoggingSetup.sysbiosTaskLogging = true;
     *  LoggingSetup.sysbiosTaskLoggingRuntimeControl = false;
     *  @p
     */
    metaonly config Bool sysbiosTaskLoggingRuntimeControl = true;

    /*!
     *  ======== overflowLoggerSize ========
     *  Size of logger used for overflow events when
     *  uploadMode is either JTAGRUNMODE or NONJTAG_AND_JTAGSTOPMODE
     */
    metaonly config SizeT overflowLoggerSize = 1024;

    /*!
     *  ========= eventUploadMode ========
     *  Event upload mode
     *
     *  See {@link LoggingSetup#UploadMode} for a description of the the
     *  different event upload modes.
     *
     *  @a(Example)
     *  This script configures the system to stream events from the
     *  target to the host over JTAG while the target is running.
     *  (Note that this mode is only supported for CPUs that
     *  support real-time JTAG accesses such as those in the C6X family,
     *  e.g. C64X+ and C66 CPUs)
     *
     *  @p(code)
     *  var LoggingSetup = xdc.useModule('ti.uia.sysbios.LoggingSetup');
     *  LoggingSetup.eventUploadMode = LoggingSetup.UploadMode_JTAGRUNMODE;
     *
     */
    metaonly config UploadMode eventUploadMode = UploadMode_JTAGSTOPMODE;

    /*!
     * ======== disableMulticoreEventCorrelation ========
     * Set to true for single core applications.
     *
     * When set to true, the LoggingSetup module will not automatically
     * include the LogSync module.  The LogSync module is
     * required in order to enable events from multiple CPU cores
     * to be correlated with each other.
     * @see ti.uia.runtime.LogSync
     *
     */
    metaonly config Bool disableMulticoreEventCorrelation = false;

    /*! @_nodoc
     * ======== createLogger =========
     * Internal helper function that creates the type of logger
     * appropriate for the LoggingSetup.uploadMode that has been configured.
     *
     * @param(loggerSize): the size of the logger in MAUs
     * @param(loggerInstanceName): the name to assign to the logger instance
     * @param (loggerPriority): the IUIATransfer.Priority to assign to the logger instance
     * @a(return) returns the logger instance that was created
     */
     metaonly function createLogger(loggerSize,loggerInstanceName,loggerPriority);
}
